@@ -27,18 +27,16 @@ namespace rfb {
 
   class CSecurityStack : public CSecurity {
   public:
-    CSecurityStack(CConnection* cc, int Type, const char *Name,
-                   CSecurity* s0 = NULL, CSecurity* s1 = NULL);
+    CSecurityStack(CConnection* cc, int Type,
+                   CSecurity* s0 = nullptr, CSecurity* s1 = nullptr);
     ~CSecurityStack();
-    virtual bool processMsg();
-    virtual int getType() const {return type;};
-    virtual const char* description() const {return name;}
-    virtual bool isSecure() const;
+    bool processMsg() override;
+    int getType() const override {return type;};
+    bool isSecure() const override;
   protected:
     int state;
     CSecurity* state0;
     CSecurity* state1;
-    const char* name;
     int type;
   };
 }
