@@ -2,7 +2,7 @@
  * Copyright (C) 2006 Constantin Kaplinsky.  All Rights Reserved.
  * Copyright (C) 2009 Paul Donohue.  All Rights Reserved.
  * Copyright (C) 2010, 2012-2013 D. R. Commander.  All Rights Reserved.
- * Copyright (C) 2011-2017 Brian P. Hinz
+ * Copyright (C) 2011-2019 Brian P. Hinz
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -135,8 +135,8 @@ class Viewport extends JPanel implements ActionListener {
   public void updateWindow() {
     Rect r = frameBuffer.getDamage();
     if (!r.is_empty()) {
-      if (cc.cp.width != scaledWidth ||
-          cc.cp.height != scaledHeight) {
+      if (cc.server.width() != scaledWidth ||
+          cc.server.height() != scaledHeight) {
         AffineTransform t = new AffineTransform(); 
         t.scale((double)scaleRatioX, (double)scaleRatioY);
         Rectangle s = new Rectangle(r.tl.x, r.tl.y, r.width(), r.height());
@@ -167,8 +167,15 @@ class Viewport extends JPanel implements ActionListener {
     for (i = 0; i < width*height; i++)
       if (data[i*4 + 3] != 0) break;
 
-    if ((i == width*height) && dotWhenNoCursor.getValue()) {
-      vlog.debug("cursor is empty - using dot");
+    useSystemCursor = false;
+    if ((i == width*height) && alwaysCursor.getValue()) {
+      String cursorTypeStr = cursorType.getValueStr();
+      if (cursorTypeStr.matches("^System$")) {
+        useSystemCursor = true;
+      } else {
+        vlog.debug("Cursor is empty: Using dot");
+      }
+      // Do this anyway to prevent cursor being null
       cursor = new BufferedImage(5, 5, BufferedImage.TYPE_INT_ARGB_PRE);
       cursor.setRGB(0, 0, 5, 5, dotcursor_xpm, 0, 5);
       cursorHotspot.x = cursorHotspot.y = 3;
@@ -188,23 +195,20 @@ class Viewport extends JPanel implements ActionListener {
       }
     }
 
-    int cw = (int)Math.floor((float)cursor.getWidth() * scaleRatioX);
-    int ch = (int)Math.floor((float)cursor.getHeight() * scaleRatioY);
-
-    int x = (int)Math.floor((float)cursorHotspot.x * scaleRatioX);
-    int y = (int)Math.floor((float)cursorHotspot.y * scaleRatioY);
-
+    int cw = (int) Math.floor((float) cursor.getWidth() * scaleRatioX);
+    int ch = (int) Math.floor((float) cursor.getHeight() * scaleRatioY);
+    int x = cursorHotspot.x;
+    int y = cursorHotspot.y;
     Dimension cs = tk.getBestCursorSize(cw, ch);
-    if (cs.width != cw && cs.height != ch) {
-      cw = Math.min(cw, cs.width);
-      ch = Math.min(ch, cs.height);
-      x = (int)Math.min(x, Math.max(cs.width - 1, 0));
-      y = (int)Math.min(y, Math.max(cs.height - 1, 0));
-      BufferedImage tmp =
-        new BufferedImage(cs.width, cs.height, BufferedImage.TYPE_INT_ARGB_PRE);
+    if (cs.width != cursor.getWidth() || cs.height != cursor.getHeight()) {
+      cw = VncViewer.os.startsWith("windows") ?  Math.min(cw, cs.width) : cs.width;
+      ch = VncViewer.os.startsWith("windows") ?  Math.min(ch, cs.height) : cs.height;
+      BufferedImage tmp = new BufferedImage(cs.width, cs.height, BufferedImage.TYPE_INT_ARGB_PRE);
       Graphics2D g2 = tmp.createGraphics();
       g2.drawImage(cursor, 0, 0, cw, ch, 0, 0, width, height, null);
       g2.dispose();
+      x = (int) Math.min(Math.floor((float) x * (float) cw / (float) width), Math.max(cw - 1, 0));
+      y = (int) Math.min(Math.floor((float) y * (float) ch / (float) height), Math.max(ch - 1, 0));
       cursor = tmp;
     }
 
@@ -219,7 +223,10 @@ class Viewport extends JPanel implements ActionListener {
 
     hotspot = new java.awt.Point(x, y);
     softCursor = tk.createCustomCursor(img, hotspot, name);
-    setCursor(softCursor);
+    if (useSystemCursor)
+      setCursor(java.awt.Cursor.getDefaultCursor());
+    else
+      setCursor(softCursor);
   }
 
   public void resize(int x, int y, int w, int h) {
@@ -240,8 +247,6 @@ class Viewport extends JPanel implements ActionListener {
     case MouseEvent.MOUSE_ENTERED:
       if (cursor != null)
         setCursor(cursor, cursorHotspot.x, cursorHotspot.y);
-      if (embed.getValue())
-        requestFocus();
       return 1;
     case MouseEvent.MOUSE_EXITED:
       setCursor(java.awt.Cursor.getDefaultCursor());
@@ -309,8 +314,8 @@ class Viewport extends JPanel implements ActionListener {
   public void paintComponent(Graphics g) {
     Graphics2D g2 = (Graphics2D)g;
     synchronized(frameBuffer.getImage()) {
-      if (cc.cp.width != scaledWidth ||
-          cc.cp.height != scaledHeight) {
+      if (cc.server.width() != scaledWidth ||
+          cc.server.height() != scaledHeight) {
         g2.setRenderingHint(RenderingHints.KEY_RENDERING,
                             RenderingHints.VALUE_RENDER_QUALITY);
         g2.drawImage(frameBuffer.getImage(), 0, 0,
@@ -342,14 +347,14 @@ class Viewport extends JPanel implements ActionListener {
         scaledWidth = width;
         scaledHeight = height;
       } else {
-        float widthRatio = (float)width / (float)cc.cp.width;
-        float heightRatio = (float)height / (float)cc.cp.height;
+        float widthRatio = (float)width / (float)cc.server.width();
+        float heightRatio = (float)height / (float)cc.server.height();
         float ratio = Math.min(widthRatio, heightRatio);
-        scaledWidth = (int)Math.floor(cc.cp.width * ratio);
-        scaledHeight = (int)Math.floor(cc.cp.height * ratio);
+        scaledWidth = (int)Math.floor(cc.server.width() * ratio);
+        scaledHeight = (int)Math.floor(cc.server.height() * ratio);
       }
-      scaleRatioX = (float)scaledWidth / (float)cc.cp.width;
-      scaleRatioY = (float)scaledHeight / (float)cc.cp.height;
+      scaleRatioX = (float)scaledWidth / (float)cc.server.width();
+      scaleRatioY = (float)scaledHeight / (float)cc.server.height();
     }
     if (scaledWidth != getWidth() || scaledHeight != getHeight())
       setSize(new Dimension(scaledWidth, scaledHeight));
@@ -360,15 +365,15 @@ class Viewport extends JPanel implements ActionListener {
     if (!viewOnly.getValue()) {
       if (buttonMask != lastButtonMask || !pos.equals(lastPointerPos)) {
         try {
-          if (cc.cp.width != scaledWidth ||
-              cc.cp.height != scaledHeight) {
+          if (cc.server.width() != scaledWidth ||
+              cc.server.height() != scaledHeight) {
             int sx = (scaleRatioX == 1.00) ?
               pos.x : (int)Math.floor(pos.x / scaleRatioX);
             int sy = (scaleRatioY == 1.00) ?
               pos.y : (int)Math.floor(pos.y / scaleRatioY);
             pos = pos.translate(new Point(sx - pos.x, sy - pos.y));
           }
-          cc.writer().pointerEvent(pos, buttonMask);
+          cc.writer().writePointerEvent(pos, buttonMask);
         } catch (Exception e) {
           vlog.error("%s", e.getMessage());
           cc.close();
@@ -379,7 +384,7 @@ class Viewport extends JPanel implements ActionListener {
     }
   }
 
-  public void handleKeyPress(int keyCode, int keySym)
+  public void handleKeyPress(long keyCode, int keySym)
   {
     // Prevent recursion if the menu wants to send it's own
     // activation key.
@@ -430,8 +435,8 @@ class Viewport extends JPanel implements ActionListener {
           downKeySym.containsValue(XK_Alt_R)) {
         vlog.debug("Faking release of AltGr (Ctrl_L+Alt_R)");
         try {
-          cc.writer().keyEvent(XK_Control_L, false);
-          cc.writer().keyEvent(XK_Alt_R, false);
+          cc.writer().writeKeyEvent(XK_Control_L, false);
+          cc.writer().writeKeyEvent(XK_Alt_R, false);
         } catch (Exception e) {
           vlog.error("%s", e.getMessage());
           cc.close();
@@ -445,14 +450,14 @@ class Viewport extends JPanel implements ActionListener {
     // and send the same on release.
     downKeySym.put(keyCode, keySym);
 
-    vlog.debug("Key pressed: 0x%04x => 0x%04x", keyCode, keySym);
+    vlog.debug("Key pressed: 0x%016x => 0x%04x", keyCode, keySym);
 
     try {
       // Fake keycode?
-      if (keyCode > 0xffff)
-        cc.writer().keyEvent(keySym, true);
+      if (keyCode > 0xffffffffL)
+        cc.writer().writeKeyEvent(keySym, true);
       else
-        cc.writer().keyEvent(keySym, true);
+        cc.writer().writeKeyEvent(keySym, true);
     } catch (Exception e) {
       vlog.error("%s", e.getMessage());
       cc.close();
@@ -464,8 +469,8 @@ class Viewport extends JPanel implements ActionListener {
           downKeySym.containsValue(XK_Alt_R)) {
         vlog.debug("Restoring AltGr state");
         try {
-          cc.writer().keyEvent(XK_Control_L, true);
-          cc.writer().keyEvent(XK_Alt_R, true);
+          cc.writer().writeKeyEvent(XK_Control_L, true);
+          cc.writer().writeKeyEvent(XK_Alt_R, true);
         } catch (Exception e) {
           vlog.error("%s", e.getMessage());
           cc.close();
@@ -474,7 +479,7 @@ class Viewport extends JPanel implements ActionListener {
     }
   }
 
-  public void handleKeyRelease(int keyCode)
+  public void handleKeyRelease(long keyCode)
   {
     Integer iter;
 
@@ -489,13 +494,13 @@ class Viewport extends JPanel implements ActionListener {
       return;
     }
 
-    vlog.debug("Key released: 0x%04x => 0x%04x", keyCode, iter);
+    vlog.debug("Key released: 0x%016x => 0x%04x", keyCode, iter);
 
     try {
-      if (keyCode > 0xffff)
-        cc.writer().keyEvent(iter, false);
+      if (keyCode > 0xffffffffL)
+        cc.writer().writeKeyEvent(iter, false);
       else
-        cc.writer().keyEvent(iter, false);
+        cc.writer().writeKeyEvent(iter, false);
     } catch (Exception e) {
       vlog.error("%s", e.getMessage());
       cc.close();
@@ -509,7 +514,7 @@ class Viewport extends JPanel implements ActionListener {
 
     if (event instanceof KeyEvent) {
       KeyEvent ev = (KeyEvent)event;
-      if (ev.getKeyCode() == 0) {
+      if (KeyMap.get_keycode_fallback_extended(ev) == 0) {
         // Not much we can do with this...
         vlog.debug("Ignoring KeyEvent with unknown Java keycode");
         return 0;
@@ -519,34 +524,41 @@ class Viewport extends JPanel implements ActionListener {
         // Generate a fake keycode just for tracking if we can't figure
         // out the proper one.  Java virtual key codes aren't unique 
         // between left/right versions of keys, so we can't use them as
-        // indexes to the downKeySym map. There should not be any key 
-        // codes > 0xFFFF so we can use the high nibble to make a unique
-        // pseudo-key code.
-        int keyCode = ev.getKeyCode() | ev.getKeyLocation()<<16;
+        // indexes to the downKeySym map.
+        long keyCode = KeyMap.get_keycode_fallback_extended(ev) | ((long)ev.getKeyLocation()<<32);
 
         // Pressing Ctrl wreaks havoc with the symbol lookup, so turn
         // that off. But AltGr shows up as Ctrl_L+Alt_R in Windows, so
         // construct a new KeyEvent that uses a proper AltGraph for the
         // symbol lookup.
-        if (VncViewer.os.startsWith("windows")) {
-          if (downKeySym.containsValue(XK_Control_L) &&
-              downKeySym.containsValue(XK_Alt_R)) {
-            int mask = ev.getModifiers();
-            mask &= ~CTRL_MASK;
-            mask &= ~ALT_MASK;
-            mask |= ALT_GRAPH_MASK;
-            AWTKeyStroke ks =
-              AWTKeyStroke.getAWTKeyStroke(ev.getKeyCode(), mask);
-            ev = new KeyEvent((JComponent)ev.getSource(), ev.getID(),
-                              ev.getWhen(), mask, ev.getKeyCode(),
-                              ks.getKeyChar(), ev.getKeyLocation());
-          }
+        int keySym;
+        if (VncViewer.os.startsWith("windows") &&
+            downKeySym.containsValue(XK_Control_L) &&
+            downKeySym.containsValue(XK_Alt_R)) {
+          int mask = ev.getModifiers();
+          mask &= ~CTRL_MASK;
+          mask &= ~ALT_MASK;
+          mask |= ALT_GRAPH_MASK;
+          AWTKeyStroke ks =
+            AWTKeyStroke.getAWTKeyStroke(KeyMap.get_keycode_fallback_extended(ev), mask);
+          // The mask manipulations above break key combinations involving AltGr
+          // and a key with an accented letter on some keyboard layouts (i.e. IT).
+          // So the code should first try the modified event, but if it returns no
+          // symbol, the original event should be used.
+          final KeyEvent winev = new KeyEvent((JComponent)ev.getSource(), ev.getID(),
+                            ev.getWhen(), mask, KeyMap.get_keycode_fallback_extended(ev),
+                            ks.getKeyChar(), ev.getKeyLocation());
+          keySym = KeyMap.vkey_to_keysym(winev);
+          if (keySym == KeyMap.NoSymbol)
+            keySym = KeyMap.vkey_to_keysym(ev);
+          else
+            ev = winev;
+        } else {
+          keySym = KeyMap.vkey_to_keysym(ev);
         }
 
-        int keySym = KeyMap.vkey_to_keysym(ev);
-
         if (keySym == KeyMap.NoSymbol)
-          vlog.error("No symbol for virtual key 0x%02x", keyCode);
+          vlog.error("No symbol for virtual key 0x%016x", keyCode);
 
         if (VncViewer.os.startsWith("linux")) {
           switch (keySym) {
@@ -578,7 +590,7 @@ class Viewport extends JPanel implements ActionListener {
 
         return 1;
       } else if (ev.getID() == KeyEvent.KEY_RELEASED) {
-        int keyCode = keyCode = ev.getKeyCode() | ev.getKeyLocation()<<16;
+        long keyCode = KeyMap.get_keycode_fallback_extended(ev) | ((long)ev.getKeyLocation()<<32);
         handleKeyRelease(keyCode);
         return 1;
       }
@@ -629,8 +641,7 @@ class Viewport extends JPanel implements ActionListener {
              this, ID.REFRESH, EnumSet.of(MENU.DIVIDER));
 
     menu_add(contextMenu, "New connection...", KeyEvent.VK_N,
-             this, ID.NEWVIEWER,
-             embed.getValue() ? EnumSet.of(MENU.INACTIVE, MENU.DIVIDER) : EnumSet.of(MENU.DIVIDER));
+             this, ID.NEWVIEWER, EnumSet.of(MENU.DIVIDER));
 
     menu_add(contextMenu, "Options...", KeyEvent.VK_O,
              this, ID.OPTIONS, EnumSet.noneOf(MENU.class));
@@ -773,7 +784,7 @@ class Viewport extends JPanel implements ActionListener {
   {
     setMenuKey();
     /*
-    setScaledSize(cc.cp.width, cc.cp.height);
+    setScaledSize(cc.server.width(), cc.server.height());
     if (!oldSize.equals(new Dimension(scaledWidth, scaledHeight))) {
     // Re-layout the DesktopWindow when the scaled size changes.
     // Ideally we'd do this with a ComponentListener, but unfortunately
@@ -808,7 +819,7 @@ class Viewport extends JPanel implements ActionListener {
   Point lastPointerPos = new Point(0, 0);
   int lastButtonMask = 0;
 
-  private class DownMap extends HashMap<Integer, Integer> {
+  private class DownMap extends HashMap<Long, Integer> {
     public DownMap(int capacity) {
       super(capacity);
     }
@@ -829,6 +840,7 @@ class Viewport extends JPanel implements ActionListener {
   float scaleRatioX, scaleRatioY;
 
   static BufferedImage cursor;
+  static boolean useSystemCursor = false;
   Point cursorHotspot = new Point();
 
 }
