@@ -18,6 +18,10 @@
 
 // -=- WMNotifier.cxx
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <rfb_win32/WMNotifier.h>
 #include <rfb_win32/WMShatter.h>
 #include <rfb_win32/MsgWindow.h>
@@ -30,7 +34,7 @@ using namespace rfb::win32;
 static LogWriter vlog("WMMonitor");
 
 
-WMMonitor::WMMonitor() : MsgWindow(_T("WMMonitor")), notifier(0) {
+WMMonitor::WMMonitor() : MsgWindow("WMMonitor"), notifier(nullptr) {
 }
 
 WMMonitor::~WMMonitor() {
